#!/usr/bin/env python
import os
import argparse
import numpy as np
import pandas as pd
import importlib
import yaml
import copy
import pysge

from zdb.modules.multirun import multidraw

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("input", help="Input file")
    parser.add_argument("cfg", help="Plotting config file")
    parser.add_argument("drawer", help="Path to drawing function")
    parser.add_argument(
        "-m", "--mode", default="multiprocessing", type=str,
        help="Parallelisation: 'multiprocessing', 'sge', 'htcondor'",
    )
    parser.add_argument(
        "-j", "--ncores", default=0, type=int,
        help="Number of cores for 'multiprocessing' jobs",
    )
    parser.add_argument(
        "--sge-opts", default="-q hep.q", type=str,
        help="SGE job options",
    )
    parser.add_argument(
        "-n", "--nplots", default=-1, type=int,
        help="Number of plots to draw. -1 = all",
    )
    parser.add_argument(
        "-o", "--outdir", default="temp", type=str, help="Output directory",
    )
    return parser.parse_args()

def rename_df_index(df, index, rename_list):
    if df is None:
        return df
    indexes = df.index.names
    tdf = df.reset_index()
    for new_val, selection in rename_list:
        tdf.loc[tdf.eval(selection),index] = new_val
    return tdf.set_index(indexes)

def parallel_draw(draw, jobs, options):
    if len(jobs)==0:
        return

    njobs = options.ncores
    if options.mode in ["multiprocessing"]:
        njobs = len(jobs)

    grouped_jobs = [list(x) for x in np.array_split(jobs, njobs)]
    tasks = [
        {"task": multidraw, "args": (draw, args), "kwargs": {}}
        for args in grouped_jobs
    ]

    if options.mode=="multiprocessing" and options.ncores==0:
        pysge.local_submit(tasks)
    elif options.mode=="multiprocessing":
        pysge.mp_submit(tasks, ncores=options.ncores)
    elif options.mode=="sge":
        pysge.sge_submit(
            "zdb-draw", "_ccsp_temp/", tasks=tasks, options=options.sge_opts,
            sleep=5, request_resubmission_options=True,
        )

def main():
    options = parse_args()

    # Setup drawer function
    module_name, function_name = options.drawer.split(":")
    draw = getattr(importlib.import_module(module_name), function_name)

    # open cfg
    with open(options.cfg, 'r') as f:
        cfg = yaml.full_load(f)

    # Read in dataframes
    df_data = pd.read_hdf(options.input, "DataAggEvents")
    df_data = df_data.loc[("central",), :]
    df_mc = pd.read_hdf(options.input, "MCAggEvents")
    df_mc = df_mc.loc[("central",), :]

    # dfs
    dfs = []
    if df_data is not None:
        dfs.append(df_data)
    if df_mc is not None:
        dfs.append(df_mc)

    # varnames
    varnames = pd.concat(dfs).index.get_level_values("varname0").unique()

    # datasets
    if df_data is not None:
        datasets = df_data.index.get_level_values("parent").unique()
    else:
        datasets = ["None"]

    # cutflows
    cutflows = pd.concat(dfs).index.get_level_values("selection").unique()

    # group into histograms
    jobs = []
    for varname in varnames:
        for dataset in datasets:
            for cutflow in cutflows:
                if varname not in cfg: continue
                job_cfg = copy.deepcopy(cfg[varname])
                job_cfg.update(cfg.get("defaults", {}))
                job_cfg.update(cfg.get(dataset+"_dataset", {}))
                job_cfg.update(cfg.get(cutflow, {}))
                job_cfg.update(cfg.get(dataset+"_dataset", {}).get(cutflow, {}))
                job_cfg.update(cfg.get(dataset+"_dataset", {}).get(cutflow, {}).get(varname, {}))
                outdir = os.path.join(options.outdir, dataset, cutflow)
                if not os.path.exists(outdir):
                    os.makedirs(outdir)
                job_cfg["outpath"] = os.path.abspath(
                    os.path.join(outdir, cfg[varname]["outpath"])
                )

                # data selection
                if df_data is None or (varname, cutflow, dataset) not in df_data.index:
                    df_data_loc = None
                else:
                    df_data_loc = df_data.loc[(varname, cutflow, dataset),:]

                # mc selection
                if df_mc is None or (varname, cutflow) not in df_mc.index:
                    df_mc_loc = None
                else:
                    df_mc_loc = df_mc.loc[(varname, cutflow),:]

                jobs.append((df_data_loc, df_mc_loc, copy.deepcopy(job_cfg)))

    if options.nplots >= 0 and options.nplots < len(jobs):
        jobs = jobs[:options.nplots]
    parallel_draw(draw, jobs, options)

if __name__ == "__main__":
    main()



import numpy as np
import pandas as pd

def df_skim(paths, cfg, outpath):
    for table_label, table_name in cfg["tables"].items():
        for path in paths:
            for df in pd.read_hdf(path, table_name, iterator=True, chunksize=500_000):
                tdf = df.query(cfg["selection"])

                tdf.loc[:, "sample"] = tdf["sample"].astype("str")
                tdf.loc[:, "parent"] = tdf["parent"].astype("str")

                tdf.to_hdf(
                    outpath, table_name, mode='a', format='table', append=True,
                    complevel=9, complib='blosc:lz4hc',
                    min_itemsize={"sample": 50, "parent": 50},
                )

    return True



#!/usr/bin/env python
import argparse
import numpy as np
import pandas as pd
import pysge
import oyaml as yaml
import functools
import tqdm

from zdb.modules.df_slim import df_slim

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("config", help="Path to yaml file")
    parser.add_argument(
        "-m", "--mode", default="multiprocessing", type=str,
        help="Parallelisation: 'multiprocessing', 'sge', 'htcondor'",
    )
    parser.add_argument(
        "-j", "--ncores", default=0, type=int,
        help="Number of cores for 'multiprocessing' jobs",
    )
    parser.add_argument(
        "--sge-opts", default="-q hep.q", type=str,
        help="Options to pass onto qsub",
    )
    parser.add_argument(
        "-n", "--nfiles", default=-1, type=int,
        help="Number of files to process. -1 = all",
    )
    parser.add_argument(
        "-o", "--output", default="output.h5", type=str, help="Output file",
    )
    return parser.parse_args()

def main():
    options = parse_args()
    mode = options.mode
    njobs = options.ncores

    # setup jobs
    with open(options.config, 'r') as f:
        cfg = yaml.full_load(f)

    # group jobs
    files = cfg["files"]
    if options.nfiles > 0:
        files = files[:options.nfiles]
    if mode in ["multiprocessing"] or njobs < 0:
        njobs = len(files)

    grouped_files = [list(x) for x in np.array_split(files, njobs)]
    tasks = [
        {"task": df_slim, "args": (fs,cfg,options.output.format(idx)), "kwargs": {}}
        for idx, fs in enumerate(grouped_files)
    ]

    if mode=="multiprocessing" and options.ncores==0:
        results = pysge.local_submit(tasks)
    elif mode=="multiprocessing":
        results = pysge.mp_submit(tasks, ncores=options.ncores)
    elif mode=="sge":
        results = pysge.sge_submit(
            "zdb", "_ccsp_temp/", tasks=tasks, options=options.sge_opts,
            sleep=5, request_resubmission_options=True,
        )
    print("Finished!")

if __name__ == "__main__":
    main()



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as ptch

def process_mc(df, cfg):
    if df is None:
        return None

    indexes = df.index.names
    df = df.reset_index()
    for old_name, new_name in cfg["rename_samples"].items():
        df.loc[df[cfg["sample"]]==old_name,cfg["sample"]] = new_name
    df = df.groupby(indexes).sum()

    binning = np.arange(*cfg["binning"])
    indexes = df.index.names
    df = df.reset_index()
    mask = (df[cfg["binvar"]] < binning[0])
    df.loc[mask,cfg["binvar"]] = 2*binning[0] - binning[1]
    df.loc[~mask,cfg["binvar"]] = binning[binning.searchsorted(df.loc[~mask,cfg["binvar"]], side='right')-1]
    df = df.groupby(indexes).sum()

    return df

def create_mc_counts_variance(df, cfg):
    if df is None:
        return None, None

    # Pivot MC tables on samples
    df_counts = pd.pivot_table(
        df, values=cfg["counts"], index=cfg["binvar"], columns=cfg["sample"],
        aggfunc='sum', fill_value=0.,
    )
    df_variance = pd.pivot_table(
        df, values=cfg["variance"], index=cfg["binvar"],
        columns=cfg["sample"], aggfunc='sum', fill_value=0.,
    )

    # Sort MC by sum across bins
    columns = df_counts.sum().sort_values().index.tolist()
    if cfg["process_at_bottom"] in columns:
        columns.remove(cfg["process_at_bottom"])
        columns = [cfg["process_at_bottom"]]+columns
    df_counts = df_counts.reindex(columns, axis=1)

    return df_counts, df_variance

def process_data(df, cfg):
    if df is None:
        return None

    binning = np.arange(*cfg["binning"])
    indexes = df.index.names
    df = df.reset_index()
    mask = (df[cfg["binvar"]] < binning[0])
    df.loc[mask,cfg["binvar"]] = 2*binning[0] - binning[1]
    df.loc[~mask,cfg["binvar"]] = binning[binning.searchsorted(df.loc[~mask,cfg["binvar"]], side='right')-1]
    df = df.groupby(indexes).sum()

    return df

def draw_data(ax, df, cfg):
    if df is None:
        return

    binning = np.arange(*cfg["binning"])
    binvars = df.index.get_level_values(cfg["binvar"])
    bincents = (binning[1:] + binning[:-1])/2.
    idx_dummy = pd.DataFrame({
        "binvar": binning[:-1], cfg["counts"]: [0]*len(binning[:-1]),
        cfg["variance"]: [0]*len(binning[:-1]),
    }).set_index("binvar")
    df = df.reindex_like(idx_dummy)

    if not cfg["blind"]:
        ax.errorbar(
            bincents, df[cfg["counts"]],
            yerr=np.sqrt(df[cfg["variance"]]), fmt='o', ms=4, lw=0.6,
            capsize=2.5, color='black', label="Data",
        )

def draw_mc_counts(ax, df, cfg):
    if df is None or df.empty or df.sum().sum()==0:
        return

    binning = np.arange(*cfg["binning"])
    binvars = df.index.get_level_values(cfg["binvar"])
    ax.hist(
        binvars, binning, weights=df.sum(axis=1),
        histtype='step', color='black', label=r'',#label=r'SM Total',
    )
    columns = df.sum().sort_values().index.tolist()
    ax.hist(
        [binvars]*df.shape[1], binning, weights=df.values,
        histtype='stepfilled', stacked=True, log=True,
        color=[cfg["process_colors"][p] for p in columns],
        label=[cfg["process_names"][p] for p in columns],
    )

def draw_legend(axt, axb, df_mc, df_data, cfg):
    handles, labels = axt.get_legend_handles_labels()

    if df_mc is not None:
        fractions = (df_mc.sum(axis=0) / df_mc.sum().sum()).values[::-1]
        fraction_labels = ["{:.3f}".format(f) for f in fractions]

        if df_data is not None and not cfg["blind"]:
            fraction_labels = [
                "{:.3f}".format(df_data[cfg["counts"]].sum().sum()/df_mc.sum().sum())
            ] + fraction_labels

            data_idx = labels.index("Data")
            data_label = labels.pop(data_idx)
            labels = [data_label]+labels
            data_handle = handles.pop(data_idx)
            handles = [data_handle]+handles

        blank_handles = [
            ptch.Rectangle((0,0), 0, 0, fill=False, edgecolor='none', visible=False)
        ]*len(fraction_labels)

        if cfg["legend_off_axes"]:
            box = axt.get_position()
            axt.set_position([box.x0, box.y0, box.width*0.8, box.height])
            axt.legend(
                handles+blank_handles, labels+fraction_labels, ncol=2,
                bbox_to_anchor=(1, 1), handleheight=1.6, labelspacing=0.05,
                columnspacing=-2,
            )
            box = axb.get_position()
            axb.set_position([box.x0, box.y0, box.width*0.8, box.height])
        else:
            axt.legend(
                handles+blank_handles, labels+fraction_labels, ncol=2,
                handleheight=0.8, labelspacing=0.05, columnspacing=-2,
            )
    else:
        handles, labels = axt.get_legend_handles_labels()
        if cfg["legend_off_axes"]:
            box = axt.get_position()
            axt.set_position([box.x0, box.y0, box.width*0.8, box.height])
            axt.legend(handles, labels, bbox_to_anchor=(1, 1))
            box = axb.get_position()
            axb.set_position([box.x0, box.y0, box.width*0.8, box.height])
        else:
            axt.legend(handles, labels)

    handles, labels = axb.get_legend_handles_labels()
    if cfg["legend_off_axes"]:
        axb.legend(handles, labels, bbox_to_anchor=(1, 1))
    else:
        axb.legned(handles, labels)

def draw_cms_header(ax):
    ax.text(
        0, 1, r'$\mathbf{CMS}\ \mathit{Preliminary}$',
        ha='left', va='bottom', transform=ax.transAxes, fontsize=12,
    )
    ax.text(
        1, 1, r'$35.9\ \mathrm{fb}^{-1}(13\ \mathrm{TeV})$',
        ha='right', va='bottom', transform=ax.transAxes, fontsize=12,
    )

def draw_ratio(ax, df, cfg):
    if df is None or cfg["blind"]:
        return

    binning = np.arange(*cfg["binning"])
    binvars = df.index.get_level_values(cfg["binvar"])
    bincents = (binning[1:] + binning[:-1])/2.
    idx_dummy = pd.DataFrame({
        "binvar": binning[:-1], "ratio": [0]*len(binning[:-1]),
        "data_err": [0]*len(binning[:-1]), "mc_err": [0]*len(binning[:-1]),
    }).set_index("binvar")
    df = df.reindex_like(idx_dummy)

    ax.errorbar(
        bincents, df["ratio"], yerr=df["data_err"], fmt='o', ms=4, lw=0.6,
        capsize=2.5, color='black', label='',
    )

    ax.fill_between(
        binning, list(1.-df["mc_err"])+[1.], list(1.+df["mc_err"])+[1.],
        step='post', color='#aaaaaa', label='MC stat. unc.',
    )

def draw_data_mc(df_data, df_mc, cfg):
    plt.style.use('cms')

    df_mc = process_mc(df_mc, cfg)
    df_mc_counts, df_mc_variance = create_mc_counts_variance(df_mc, cfg)
    df_data = process_data(df_data, cfg)

    fig, (axt, axb) = plt.subplots(
        figsize = (4.8, 6), nrows=2, ncols=1, sharex='col', sharey=False,
        gridspec_kw={'height_ratios': [2.5, 1], 'wspace': 0.1, 'hspace': 0.1},
    )

    draw_data(axt, df_data, cfg)
    draw_mc_counts(axt, df_mc_counts, cfg)

    if df_mc is not None and df_data is not None:
        df_ratio = pd.DataFrame({
            "ratio": df_data["sum_w"] / df_mc_counts.sum(axis=1),
            "data_err": np.sqrt(df_data["sum_ww"]) / df_mc_counts.sum(axis=1),
            "mc_err": np.sqrt(df_mc_variance.sum(axis=1)) / df_mc_counts.sum(axis=1),
        }, index=df_mc_counts.index)
    else:
        df_ratio = None
    draw_ratio(axb, df_ratio, cfg)

    draw_cms_header(axt)
    draw_legend(axt, axb, df_mc_counts, df_data, cfg)

    ylims = axt.get_ylim()
    binning = np.arange(*cfg["binning"])
    axt.set_xlim(binning[0], binning[-1])
    axt.set_ylim(max(ylims[0], 0.5), ylims[1])
    axt.set_ylabel("Number of events", fontsize=12)

    ylims = axb.get_ylim()
    axb.set_ylim(max(ylims[0], 0.5), min(ylims[1], 1.5))
    axb.set_xlabel(cfg["label"], fontsize=12)
    axb.set_ylabel("Data/Simulation", fontsize=12)

    axb.axhline(1., lw=0.8, ls='--', color='black')

    #print("Creating {}".format(cfg["outpath"]))
    fig.align_ylabels([axt, axb])
    fig.savefig(cfg["outpath"], format="pdf", bbox_inches="tight")
    plt.close(fig)

    return True



#!/usr/bin/env python
import argparse
from zdb.modules.analyse import analyse

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("config", help="Path to yaml file")
    parser.add_argument(
        "-m", "--mode", default="multiprocessing", type=str,
        help="Parallelisation: 'multiprocessing', 'sge', 'condor'",
    )
    parser.add_argument(
        "-j", "--ncores", default=0, type=int,
        help="Number of cores for 'multiprocessing' jobs",
    )
    parser.add_argument(
        "--batch-opts", default="", type=str,
        help="Options to pass onto the batch",
    )
    parser.add_argument(
        "-n", "--nfiles", default=-1, type=int,
        help="Number of files to process. -1 = all",
    )
    parser.add_argument(
        "--merge-locally", default=False, action='store_true',
        help="Merge locally (otherwise on the batch)",
    )
    parser.add_argument(
        "-o", "--output", default="output.pkl", type=str, help="Output file",
    )
    return parser.parse_args()

if __name__ == "__main__":
    options = parse_args()
    analyse(**options)



#!/usr/bin/env python
import os
import re
import argparse
import pandas as pd
import importlib
import yaml
import copy

from zdb.modules.multirun import multidraw
from atsge.build_parallel import build_parallel

import logging
logging.getLogger(__name__).setLevel(logging.INFO)
logging.getLogger("atsge.SGEJobSubmitter").setLevel(logging.INFO)
logging.getLogger("atsge.WorkingArea").setLevel(logging.INFO)

logging.getLogger(__name__).propagate = False
logging.getLogger("atsge.SGEJobSubmitter").propagate = False
logging.getLogger("atsge.WorkingArea").propagate = False

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("mc", help="Path to MC pandas pickle")
    parser.add_argument("drawer", help="Path to drawing function")
    parser.add_argument("cfg", help="Plotting config file")
    parser.add_argument(
        "-m", "--mode", default="multiprocessing", type=str,
        help="Parallelisation: 'multiprocessing', 'sge', 'htcondor'",
    )
    parser.add_argument(
        "-j", "--ncores", default=0, type=int,
        help="Number of cores for 'multiprocessing' jobs",
    )
    parser.add_argument(
        "-n", "--nplots", default=-1, type=int,
        help="Number of plots to draw. -1 = all",
    )
    parser.add_argument(
        "-o", "--outdir", default="temp", type=str, help="Output directory",
    )
    return parser.parse_args()

def rename_df_index(df, index, rename_list):
    if df is None:
        return df
    indexes = df.index.names
    tdf = df.reset_index()
    for new_val, selection in rename_list:
        tdf.loc[tdf.eval(selection),index] = new_val
    return tdf.set_index(indexes)

def parallel_draw(draw, jobs, options):
    if len(jobs)==0:
        return

    njobs = options.ncores
    if options.mode in ["multiprocessing"]:
        njobs = len(jobs)+1

    jobs = [
        jobs[i:i+len(jobs)//njobs+1]
        for i in xrange(0, len(jobs), len(jobs)//njobs+1)
    ]

    parallel = build_parallel(
        options.mode, processes=options.ncores, quiet=False,
        dispatcher_options={"vmem": 6, "walltime": 3*60*60},
    )
    parallel.begin()
    try:
        parallel.communicationChannel.put_multiple([{
            'task': multidraw,
            'args': (draw, args),
            'kwargs': {},
        } for args in jobs])
        results = parallel.communicationChannel.receive()
    except KeyboardInterrupt:
        parallel.terminate()
    parallel.end()

def main():
    options = parse_args()

    # Setup drawer function
    module_name, function_name = options.drawer.split(":")
    draw = getattr(importlib.import_module(module_name), function_name)

    # open cfg
    with open(options.cfg, 'r') as f:
        cfg = yaml.load(f)

    # Read in dataframes
    df = pd.read_pickle(options.mc) if options.mc is not None else None
    df = rename_df_index(df, "parent", [
        ("WJetsToENu", "(parent=='WJetsToLNu') & (LeptonIsElectron==1)"),
        ("WJetsToMuNu", "(parent=='WJetsToLNu') & (LeptonIsMuon==1)"),
        #("WJetsToTauNu", "(parent=='WJetsToLNu') & (LeptonIsTau==1)"),
        ("WJetsToTauHNu", "(parent=='WJetsToLNu') & (LeptonIsTau==1) & (nGenTauL==0)"),
        ("WJetsToTauLNu", "(parent=='WJetsToLNu') & (LeptonIsTau==1) & (nGenTauL==1)"),
        ("DYJetsToEE", "(parent=='DYJetsToLL') & (LeptonIsElectron==1)"),
        ("DYJetsToMuMu", "(parent=='DYJetsToLL') & (LeptonIsMuon==1)"),
        #("DYJetsToTauTau", "(parent=='DYJetsToLL') & (LeptonIsTau==1)"),
        ("DYJetsToTauHTauH", "(parent=='DYJetsToLL') & (LeptonIsTau==1) & (nGenTauL==0)"),
        ("DYJetsToTauHTauL", "(parent=='DYJetsToLL') & (LeptonIsTau==1) & (nGenTauL==1)"),
        ("DYJetsToTauLTauL", "(parent=='DYJetsToLL') & (LeptonIsTau==1) & (nGenTauL==2)"),
    ]).reset_index(["LeptonIsElectron", "LeptonIsMuon", "LeptonIsTau", "nGenTauL"], drop=True)
    df = df.groupby(df.index.names).sum()

    # cutflows
    cutflows = df.index.get_level_values("selection").unique()

    # variations
    varnames = df.index.get_level_values("varname").unique()
    regex = re.compile("^(?P<varname>[a-zA-Z0-9_]+)_(?P<variation>[a-zA-Z0-9]+)(Up|Down)$")

    varname_variations = {}
    for v in varnames:
        match = regex.search(v)
        if match:
            varname = match.group("varname")
            variation = match.group("variation")
            if varname not in varname_variations:
                varname_variations[varname] = []
            if variation not in varname_variations[varname]:
                varname_variations[varname].append(variation)

    # group into histograms
    jobs = []
    for cutflow in cutflows:
        for varname, variations in varname_variations.items():
            for variation in variations:
                job_cfg = copy.deepcopy(cfg[variation])
                job_cfg.update(cfg.get("defaults", {}))
                job_cfg.update(cfg.get(cutflow, {}))
                job_cfg.update(cfg.get(varname, {}))
                outdir = os.path.join(options.outdir, cutflow, varname)
                if not os.path.exists(outdir):
                    os.makedirs(outdir)
                job_cfg["outpath"] = os.path.abspath(
                    os.path.join(outdir, cfg[variation]["outpath"])
                )

                df_loc = df.loc[
                    (
                        (df.index.get_level_values("selection")==cutflow)
                        & (df.index.get_level_values("varname").isin(
                            [varname+"_nominal", varname+"_"+variation+"Up", varname+"_"+variation+"Down"]
                        ))
                    ), :
                ]
                jobs.append((df_loc, copy.deepcopy(job_cfg)))

    if options.nplots >= 0 and options.nplots < len(jobs):
        jobs = jobs[:options.nplots]
    parallel_draw(draw, jobs, options)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import argparse
import numpy as np
import pandas as pd
import pysge
import glob
import os

from zdb.modules.df_process import df_merge, df_open_merge

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("path", help="Path to temp dir")
    parser.add_argument(
        "-m", "--mode", default="multiprocessing", type=str,
        help="Parallelisation: 'multiprocessing', 'sge', 'htcondor'",
    )
    parser.add_argument(
        "-j", "--ncores", default=0, type=int,
        help="Number of cores for 'multiprocessing' jobs",
    )
    parser.add_argument(
        "--sge-opts", default="-q hep.q", type=str,
        help="Options to pass onto qsub",
    )
    parser.add_argument(
        "-o", "--output", default="output.pkl", type=str, help="Output file",
    )
    return parser.parse_args()

def main():
    options = parse_args()

    results = pysge.sge_resume(
        "zdb", options.path, options=options.sge_opts, sleep=5,
        request_resubmission_options=True,
    )

    njobs = options.ncores
    if options.mode in ["multiprocessing"] or options.ncores < 0:
        njobs = len(results)

    grouped_args = [list(x) for x in np.array_split(results, njobs)]
    tasks = [
        {"task": df_open_merge, "args": (args,), "kwargs": {"quiet": True}}
        for args in grouped_args
    ]

    if options.mode=="multiprocessing" and options.ncores==0:
        merge_results = pysge.local_submit(tasks)
        df = pd.DataFrame()
        for result in merge_results:
            df = df_merge(df, result)
    elif options.mode=="multiprocessing":
        merge_results = pysge.mp_submit(tasks, ncores=options.ncores)
        df = pd.DataFrame()
        for result in merge_results:
            df = df_merge(df, result)
    elif options.mode=="sge":
        merge_results = pysge.sge_submit(
            "zdb-merge", "_ccsp_temp/", tasks=tasks, options=options.sge_opts,
            sleep=5, request_resubmission_options=True,
        )
        df = df_open_merge(merge_results)
    else:
        df = pd.DataFrame()

    print(df)
    path, table = options.output.split(":")
    df.to_hdf(
            path, table, format='table', append=False, complevel=9,
        complib='blosc:lz4hc',
    )

if __name__ == "__main__":
    main()



import numpy as np
import pandas as pd
import pysge
import yaml
import functools

from zdb.modules.df_process import df_process, df_merge, df_open_merge

def analyse(
    config, mode="multiprocesing", ncores=0, nfiles=-1, batch_opts="",
    output=None, merge_locally=True,
):
    njobs = ncores

    # setup jobs
    with open(config, 'r') as f:
        cfg = yaml.load(f)

    # group jobs
    files = cfg["files"]
    if nfiles > 0:
        files = files[:nfiles]
    if mode in ["multiprocessing"] or njobs < 0:
        njobs = len(files)

    grouped_files = [list(x) for x in np.array_split(files, njobs)]
    tasks = [
        {"task": df_process, "args": (fs, cfg["query"]), "kwargs": {}}
        for fs in grouped_files
    ]

    if mode=="multiprocessing" and ncores==0:
        results = pysge.local_submit(tasks)
        df = functools.reduce(lambda x, y: df_merge(x, y), results)
    elif mode=="multiprocessing":
        results = pysge.mp_submit(tasks, ncores=ncores)
        df = functools.reduce(lambda x, y: df_merge(x, y), results)
    elif mode=="sge":
        results = pysge.sge_submit(
            "zdb", "_ccsp_temp/", tasks=tasks, options=batch_opts,
            sleep=5, request_resubmission_options=True,
        )

        grouped_args = [list(x) for x in np.array_split(results, 50)]
        tasks = [
            {"task": df_open_merge, "args": (args,), "kwargs": {"quiet": True}}
            for args in grouped_args
        ]
        if merge_locally:
            merge_results = results[:]
        else:
            merge_results = pysge.sge_submit(
                "zdb-merge", "_ccsp_temp/", tasks=tasks, options=batch_opts,
                sleep=5, request_resubmission_options=True,
            )
        df = df_open_merge(merge_results)
    elif mode=="condor":
        import conpy
        results = conpy.condor_submit(
            "zdb", "_ccsp_temp/", tasks=tasks, options=batch_opts,
            sleep=5, request_resubmission_options=True,
        )

        grouped_args = [list(x) for x in np.array_split(results, 50)]
        tasks = [
            {"task": df_open_merge, "args": (args,), "kwargs": {"quiet": True}}
            for args in grouped_args
        ]
        if merge_locally:
            merge_results = results[:]
        else:
            merge_results = conpy.condor_submit(
                "zdb-merge", "_ccsp_temp/", tasks=tasks, options=batch_opts,
                sleep=5, request_resubmission_options=True,
            )
        df = df_open_merge(merge_results)
    else:
        df = pd.DataFrame()

    #print(df)
    if output is not None:
        path, table = output.split(":")
        df.to_hdf(
            path, table, format='table', append=False, complevel=9,
            complib='blosc:lz4hc',
        )
    else:
        return df




import numpy as np
import pandas as pd
import oyaml as yaml
import pysge
from zdb.modules.df_skim import df_skim

def skim(
    config, mode="multiprocessing", ncores=0, nfiles=-1, batch_opts="",
    output=None,
):
    njobs = ncores

    #setup jobs
    with open(config, 'r') as f:
        cfg = yaml.load(f)

    # group jobs
    files = cfg["files"]
    if nfiles > 0:
        files = files[:nfiles]
    if mode in ["multiprocessing"] or njobs < 0:
        njobs = len(files)

    grouped_files = [list(x) for x in np.array_split(files, njobs)]
    tasks = [
        {"task": df_skim, "args": (fs, cfg, output.format(idx)), "kwargs": {}}
        for fs in grouped_files
    ]

    if mode=="multiprocessing" and ncores==0:
        results = pysge.local_submit(tasks)
    elif mode=="multiprocessing":
        results = pysge.mp_submit(tasks, ncores=ncores)
    elif mode=="sge":
        results = pysge.sge_submit(
            "zdb", "_ccsp_temp/", tasks=tasks, options=batch_opts,
            sleep=5, request_resubmission_options=True,
        )
    elif mode=="condor":
        import conpy
        results = conpy.condor_submit(
            "zdb", "_ccsp_temp/", tasks=tasks, options=batch_opts,
            sleep=5, request_resubmission_options=True,
        )
    print("Finished!")



import lz4.frame
import pickle
import numpy as np
import pandas as pd
import copy
from tqdm.auto import tqdm

def df_merge(df1, df2):
    if df1 is None or df1.empty:
        return df2
    if df2 is None or df2.empty:
        return df1

    reindex = df1.index.union(df2.index)
    return df1.reindex(reindex).fillna(0.) + df2.reindex(reindex).fillna(0.)

def df_open_merge(paths, quiet=False):
    pbar = tqdm(total=len(paths), desc="Merged", dynamic_ncols=True, disable=quiet)
    obj_out = pd.DataFrame()
    for path in paths:
        with lz4.frame.open(path, 'rb') as f:
            obj_in = pickle.load(f)
        obj_out = df_merge(obj_out, obj_in)
        pbar.update()
    pbar.close()
    return obj_out

def df_process(paths, cfg):
    out_df = pd.DataFrame()

    for path in paths:
        store = pd.HDFStore(path)
        for table_label, table_name in cfg["tables"].items():
            hist_cfg = {"table_name": table_label}

            for df in store.select(table_name, iterator=True, chunksize=100000):

                # pre-eval
                for evs in cfg["eval"]:
                    for key, val in evs.items():
                        df[key] = eval("lambda "+val)(df)

                for cutflow_name, cutflow_cfg in cfg["cutflows"].items():
                    hist_cfg.update(cutflow_cfg)

                    # apply selection
                    sdf = df.loc[df.eval(cutflow_cfg["selection"])]

                    for hist_label in cutflow_cfg["hists"]:
                        hdf = sdf.copy()

                        # hist evals
                        evals = cfg["hists"][hist_label]
                        for evs in evals:
                            for key, val in evs.items():
                                hdf[key] = eval("lambda "+val.format(**hist_cfg))(hdf)

                        # add hist
                        columns = [list(ev.keys())[0] for ev in evals]
                        out_df = df_merge(
                            out_df,
                            hdf.loc[:,columns].groupby(cfg["groupby"]).sum(),
                        )
        store.close()
    return out_df



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as ptch

def process_dataframe(df, cfg):
    indexes = df.index.names
    df = df.reset_index()
    for old_name, new_name in cfg["rename_samples"].items():
        df.loc[df[cfg["sample"]]==old_name,cfg["sample"]] = new_name
    df = df.groupby(indexes).sum()

    binning = np.arange(*cfg["binning"])
    indexes = df.index.names
    df = df.reset_index()
    mask = (df[cfg["binvar"]] < binning[0])
    df.loc[mask,cfg["binvar"]] = 2*binning[0] - binning[1]
    df.loc[~mask,cfg["binvar"]] = binning[binning.searchsorted(df.loc[~mask,cfg["binvar"]], side='right')-1]
    df = df.groupby(indexes).sum()

    return df

def create_mc_counts_variance(df, cfg):
    # Pivot MC tables on samples
    df_counts = pd.pivot_table(
        df, values=cfg["counts"], index=cfg["binvar"],
        columns=[cfg["sample"], cfg["varname"]], aggfunc='sum', fill_value=0.,
    )
    df_variance = pd.pivot_table(
        df, values=cfg["variance"], index=cfg["binvar"],
        columns=[cfg["sample"], cfg["varname"]], aggfunc='sum', fill_value=0.,
    )

    new_cols = []
    for c in df_counts.columns.levels[1]:
        new_cols.append(
            "Up" if c.endswith("Up") else
            "Down" if c.endswith("Down") else
            "Nominal"
        )
    df_counts.columns = df_counts.columns.set_levels([df_counts.columns.levels[0], new_cols])
    df_variance.columns = df_variance.columns.set_levels([df_counts.columns.levels[0], new_cols])

    return df_counts, df_variance

def draw_ratios(ax, df, cfg):
    binning = np.arange(*cfg["binning"])
    binvars = df.index.get_level_values(cfg["binvar"]).tolist()
    labels = [c[0] for c in df.columns.to_flat_index()]
    ax.hist(
        [binvars]*df.shape[1], bins=binning, weights=df.values, histtype='step',
        color=[cfg["process_colors"].get(c, c) for c in labels],
        label=[cfg["process_names"].get(labels[idx], labels[idx]) if idx%2==0 else "" for idx in range(len(labels))],
    )

def draw_legend(ax):
    handles, labels = ax.get_legend_handles_labels()
    return ax.legend(handles, labels)

def draw_cms_header(ax):
    ax.text(
        0, 1, r'$\mathbf{CMS}\ \mathit{Preliminary}$',
        ha='left', va='bottom', transform=ax.transAxes, fontsize=12,
    )
    ax.text(
        1, 1, r'$35.9\ \mathrm{fb}^{-1}(13\ \mathrm{TeV})$',
        ha='right', va='bottom', transform=ax.transAxes, fontsize=12,
    )

def draw_variation(df, cfg):
    plt.style.use('cms')

    df = process_dataframe(df, cfg)
    df_counts, df_variance = create_mc_counts_variance(df, cfg)

    dfs = []
    for c in df_counts.columns.levels[0]:
        dfs.append(
            df_counts.loc[:, (c, ("Up", "Down"))]
            .divide(df_counts.loc[:, (c, "Nominal")], axis='index')
            - 1.
        )
    df_ratio = (
        pd.concat(dfs, axis='columns')
        .replace([np.inf, -np.inf], np.nan)
        .fillna(0.)
    )

    fig, ax = plt.subplots(figsize = (5.4, 4.8))

    draw_ratios(ax, df_ratio, cfg)
    draw_cms_header(ax)
    draw_legend(ax)

    ylim = min(0.5, max(map(abs, ax.get_ylim())))
    ax.set_ylim(-ylim, +ylim)
    ax.axhline(1., ls='--', lw=0.8, color='black')

    print("Creating {}".format(cfg["outpath"]))
    fig.savefig(cfg["outpath"], format="pdf", bbox_inches="tight")
    plt.close(fig)

    return True



import numpy as np
import pandas as pd

def df_slim(paths, cfg, outpath):
    for table_label, table_name in cfg["tables"].items():
        for path in paths:
            for df in pd.read_hdf(path, table_name, iterator=True, chunksize=500_000):
                tdf = pd.DataFrame()

                for evs in cfg["columns"]:
                    for key, val in evs.items():
                        print(val)
                        print(df.eval(val))
                        tdf[key] = df.eval(val, engine='python')

                tdf.to_hdf(
                    outpath, table_name, mode='a', format='table', append=True,
                    complevel=9, complib='blosc:lz4hc',
                )

    return True
